#include <sys/types.h> /* pid_t */
#include <sys/wait.h>
#include <unistd.h>    /* fork */
#include <stdio.h>
#include <stdlib.h>    /* exit */
#define ARRAY_DIM (1 << LOG_ARRAY_DIM)

int main()
{
	int array[ARRAY_DIM];
	int sum = 0;
	unsigned i;
	pid_t pid;
	
	for (i = 0; i < ARRAY_DIM; i++) /* array riempito con i valori crescenti 1..ARRAY_DIM */
		array[i] = i + 1;
	for (i = 0; i < ARRAY_DIM; i++) /* modo ovvio di calcolarne la somma */
		sum += array[i];
	printf("Sum is %d\n", sum);
	{	/* calcolo della somma spezzandola in due parti, ma computazione effettuata da un solo processo */
		int sum2 = 0;
		sum = 0;
		for (i = 0; i < ARRAY_DIM/2; i++)
			sum2 += array[i];
		for (i = ARRAY_DIM/2; i < ARRAY_DIM; i++)
			sum += array[i];
		printf("Sum is %d = %d + %d\n", sum + sum2, sum2, sum);
	}
	sum = 0;
	pid = fork();
	/* 
		calcolo della somma spezzandola in due parti: il figlio fa la prima parte, il padre la seconda, 
		quindi la computazione e' su due processi
	*/
	if (pid == 0) { /* figlio */
#ifdef DEBUG 
		/* per debuggare il figlio: 
			compilare con -DDEBUG, lanciare, vedere il pid del figlio 
			gdb for_fork <pid_del_figlio> 
			andare con l'istruzione gdb "next" fino al while
			set tmptmptmp = 0
			con un'altra next si uscira' dal while
			da questo momento in poi, si puo' debuggare il figlio
		*/
		int tmptmptmp = 1;
		while (tmptmptmp)
			tmptmptmp = 1;
#endif
		for (i = 0; i < ARRAY_DIM/2; i++)
			sum += array[i];
#ifdef DELAY_CHILD
		sleep(DELAY_CHILD);
#endif
		printf("returning %d\n", sum);
		exit(sum);
	}
	else if (pid > 0) { /* padre */
		int sum2;
		for (i = ARRAY_DIM/2; i < ARRAY_DIM; i++)
			sum += array[i];
#ifdef DELAY_PARENT
		sleep(DELAY_PARENT);
#endif
#ifndef NO_WAIT
		pid = wait(&sum2); /* si blocca finche' il figlio non finisce */
#else
		pid = waitpid(-1, &sum2, WNOHANG);	/* se il figlio ha finito (normalmente o no), 
							prende il risultato, altrimenti va avanti */
#endif
		if (pid > 0 && WIFEXITED(sum2)) 
			printf("Sum is %d = %d + %d\n", sum + WEXITSTATUS(sum2), WEXITSTATUS(sum2), sum);
		else if (!pid)
			printf("child not terminated\n");
		else if (pid < 1)
#ifndef NO_WAIT
			perror("error on wait\n");
#else
			perror("error on waitpid\n");
#endif
		else
			printf("child %d terminated abnormally\n", pid);
	}
	else
		perror("Could not fork");
#ifdef FINAL_DELAY_PARENT
		sleep(FINAL_DELAY_PARENT);
#endif
	return 1;
}
